const Backbone = require("backbone");
const _ = require("underscore");
const Chart = require("chart.js");
const { ipcRenderer } = require("electron");
const {
    SessionImageModel,
    SessionSettingsModel,
    IntensityDatasetModel,
    TransmittanceDatasetModel,
    SessionComponentCollection,
    DatasetCollection
} = require("./models.js");

function myFunction() {
  ipcRenderer.send("download-file","none",1);

}

let SessionImageView = Backbone.View.extend({
    template: _.template(Backbone.$("#tmpl-img-component").html()),

    initialize: function () {
        let view = this;

        ipcRenderer.on("open-file:" + this.model.url(), function (event, path) {
            view.trigger("open-file", path);
        });

        this.on("open-file", function (path) {
            view.model.set({
                path: path,
                blueX: null,
                blueY: null,
                redX: null,
                redY: null
            });
            view.model.save();
        });

        this.listenTo(this.model, "sync change", this.render);
        this.render();
    },

    events: {
        "click .img-open-button": function (e) {
            ipcRenderer.send("open-file", this.model.url());
        },
        "click .img-edit-button": function (e) {
            ipcRenderer.send("editor-open", this.model.toJSON());
        }
    },

    render: function () {
        this.$el.html(this.template(this.model.toJSON()));
        return this;
    }
});



let SessionSettingsView = Backbone.View.extend({
    template: _.template(Backbone.$("#tmpl-settings-component").html()),

    initialize: function () {
        this.render();
    },

    render: function () {
        this.$el.html(this.template(this.model.toJSON()));
        return this;
    },

    events: {
        "change .settings-specwidth": function (e) {
            let $input = this.$(e.target);

            if (e.target.checkValidity()) {
                this.model.set("specWidth", Number($input.val()));
                this.model.save();
            } else {
                $input.val(this.model.get("specWidth"));
            }
        }
    }
});

Chart.defaults.global.maintainAspectRatio = false;
Chart.defaults.global.elements.line.tension = 0.0;
Chart.defaults.global.elements.line.borderWidth = 1;
Chart.defaults.global.elements.point.radius = 0.0;
Chart.defaults.global.tooltips.mode = "index";
Chart.defaults.global.tooltips.intersect = false;
Chart.defaults.global.tooltips.cornerRadius = 2;
Chart.defaults.global.hover.mode = "index";
Chart.defaults.global.hover.intersect = false;
Chart.defaults.global.hover.animationDuration = 200;
Chart.defaults.global.defaultFontSize = 14;
Chart.defaults.global.defaultFontFamily = "'BlinkMacSystemFont', 'Lucida Grande', 'Segoe UI', Ubuntu, Cantarell, sans-serif";
Chart.defaults.global.animation.easing = "easeOutExpo";

let IntensitySpectrumView = Backbone.View.extend({
    template: _.template(Backbone.$("#tmpl-chart").html()),
    className: "pure-u-1 pure-u-xl-1-2",
    chart: null,

    initialize: function () {
        this.listenTo(this.model, "sync change", this.render);
        this.render();
    },

    render: function () {
        if (this.chart) {
            this.chart.destroy();
        }
        this.$el.html(this.template(this.model.attributes));
        this.chart = new Chart(this.$("canvas"), {
            type: "line",
            data: {
                labels: this.model.get("nm"),
                datasets: [
                    {
                        label: "Red",
                        borderColor: "rgba(255, 50, 0, 1)",
                        backgroundColor: "rgba(255, 50, 0, 0.02)",
                        data: this.model.get("r")

                    },
                    {
                        label: "Green",
                        borderColor: "rgba(60, 230, 0, 1)",
                        backgroundColor: "rgba(60, 230, 0, 0.02)",
                        data: this.model.get("g")
                    },
                    {
                        label: "Blue",
                        borderColor: "rgba(0, 90, 255, 1)",
                        backgroundColor: "rgba(0, 90, 255, 0.02)",
                        data: this.model.get("b")
                    },
                    {
                        label: "Total",
                        borderColor: "rgba(40, 40, 40, 1)",
                        backgroundColor: "rgba(40, 40, 40, 0.02)",
                        data: this.model.get("sum")
                    }
                ]
            },
            options: {
                scales: {
                    xAxes: [{
                        ticks: {
                            autoSkipPadding: 36
                        }
                    }]
                }
            }
        });
        return this;
    }
});

let TransmittanceSpectrumView = Backbone.View.extend({
    template: _.template(Backbone.$("#tmpl-chart").html()),
    className: "pure-u-1",
    chart: null,

    initialize: function () {
        this.listenTo(this.model, "sync change", this.render);
        this.render();
    },

    render: function () {
        if (this.chart) {
            this.chart.destroy();
        }
        this.$el.html(this.template(this.model.attributes));
        this.chart = new Chart(this.$("canvas"), {
            type: "line",
            data: {
                labels: this.model.get("nm"),
                datasets: [
                    {
                        label: "Transmittance",
                        borderColor: "rgba(255, 120, 40, 1)",
                        backgroundColor: "rgba(255, 120, 40, 0.02)",
                        data: this.model.get("t")
                    },
                    {
                        label: "Absorbance",
                        borderColor: "rgba(120, 30, 255, 1)",
                        backgroundColor: "rgba(120, 30, 255, 0.02)",
                        data: this.model.get("a")
                    }
                ]
            },
            options: {
                scales: {
                    xAxes: [{
                        ticks: {
                            autoSkipPadding: 36
                        }
                    }]
                }
            }
        });
        return this;
    }
});

let SessionView = Backbone.View.extend({
    el: "#session",

    initialize: function () {
        this.collection.each(function (model) {
            let view;
            switch (model.get("type")) {
                case "image":
                    view = new SessionImageView({model: model});
                    break;
                case "settings":
                    view = new SessionSettingsView({model: model});
                    break;
                default:
                    break;
            }
            this.$el.append(view.$el);
        }, this);
    }
});

let SpectraView = Backbone.View.extend({
    el: "#spectra",

    initialize: function () {
        this.collection.each(function (model) {
            let view;
            switch (model.get("type")) {
                case "intensity":
                    view = new IntensitySpectrumView({model: model});
                    break;
                case "transmittance":
                    view = new TransmittanceSpectrumView({model: model});
                default:
                    break;
            }
            this.$el.append(view.$el);
        }, this);
    }
});

let aboutToggle = function () {
    if (spectra.findIndex(function (spectrum) {
        return spectrum.get("nm").length != 0;
    }) == -1) {
        Backbone.$("#about-overlay").fadeIn("fast");
    } else {
        Backbone.$("#about-overlay").fadeOut("fast");
    }
}

// Instantiate an absorption session

let session = new SessionComponentCollection();
session.fetch().then(function () {
    let sessionView = new SessionView({collection: session});
});

let spectra = new DatasetCollection();
spectra.fetch().then(function () {
    aboutToggle();
    let spectraView = new SpectraView({collection: spectra});
});


ipcRenderer.on("data-change", function (event) {
    spectra.fetch().then(function () {
        aboutToggle();
    });
});
