const Backbone = require("backbone");
const _ = require("underscore");
const { ipcRenderer } = require("electron");
const uuid = require("uuid/v1");

let rendererSync = function (method, model, options) {
    let deferred = Backbone.$.Deferred();
    let channel = uuid();

    switch (method) {
        case "read":
            ipcRenderer.once(channel, function (event, resp) {
                if (resp === "404") {
                    deferred.reject();
                } else {
                    deferred.resolve(resp);
                }
            });
            ipcRenderer.send("backbone-get", _.result(model, "url"), channel);
            break;
        case "create":
        case "update":
        case "patch":
            ipcRenderer.once(channel, function (event) {
                deferred.resolve();
            });
            ipcRenderer.send("backbone-set", _.result(model, "url"), channel, model.toJSON(options));
            break;
        case "delete":
            ipcRenderer.once(channel, function (event) {
                deferred.resolve();
            });
            ipcRenderer.send("backbone-delete", _.result(model, "url"), channel);
            break;
    }

    deferred.then(options.success, options.error);
    model.trigger("request", model, deferred, options);
    return deferred.promise();
}

let RendererSyncModelBase = Backbone.Model.extend({
    sync: rendererSync
});

let RendererSyncCollectionBase = Backbone.Collection.extend({
    sync: rendererSync
});

let SessionComponentBase = RendererSyncModelBase.extend({
    defaults: {
        type: "!base"
    },

    isReady: function () {
        return true;
    }
});

module.exports.SessionImageModel = SessionComponentBase.extend({
    defaults: _.extend({}, SessionComponentBase.prototype.defaults, {
        type: "image",
        title: "Image",
        path: null,
        blueNM: 400,
        blueX: null,
        blueY: null,
        redNM: 720,
        redX: null,
        redY: null
    }),

    isReady: function () {
        let check = ["path", "blueNM", "blueX", "blueY", "redNM", "redX", "redY"];
        return _.every(check, function (c) { return this.has(c); }, this);
    }
});

module.exports.SessionSettingsModel = SessionComponentBase.extend({
    defaults: _.extend({}, SessionComponentBase.prototype.defaults, {
        type: "settings",
        specWidth: 9
    })
});

module.exports.SessionComponentCollection = RendererSyncCollectionBase.extend({
    url: "/session",

    model: function (attrs, options) {
        switch (attrs.type) {
            case "image":
                return new SessionImageModel(attrs, options);
                break;
            case "settings":
                return new SessionSettingsModel(attrs, options);
                break;
            default:
                return new SessionComponentBase(attrs, options);
                break;
        }
    }
});

let DatasetModelBase = RendererSyncModelBase.extend({
    defaults: {
        type: "!base",
        title: "Dataset"
    }
});

module.exports.IntensityDatasetModel = DatasetModelBase.extend({
    defaults: _.extend({}, DatasetModelBase.prototype.defaults, {
        type: "intensity",
        title: "Intensity",
        nm: [],
        i: [],
        r: [],
        g: [],
        b: [],
        sum: []
    })
});

module.exports.TransmittanceDatasetModel = DatasetModelBase.extend({
    defaults: _.extend({}, DatasetModelBase.prototype.defaults, {
        type: "transmittance",
        title: "Transmittance",
        nm: [],
        t: []
    })
});

module.exports.DatasetCollection = RendererSyncCollectionBase.extend({
    url: "/data",

    model: function (attrs, options) {
        switch (attrs.type) {
            case "intensity":
                return new IntensityDatasetModel(attrs, options);
                break;
            case "transmittance":
                return new TransmittanceDatasetModel(attrs, options);
                break;
            default:
                return new DatasetModelBase(attrs, options);
                break;
        }
    }
});