const Jimp = require("jimp");

let pixelScale = function (pixel, c) {
    return {
        r: c * pixel.r,
        g: c * pixel.g,
        b: c * pixel.b
    }
}

let pixelAdd = function (pixA, pixB) {
    return {
        r: pixA.r + pixB.r,
        g: pixA.g + pixB.g,
        b: pixA.b + pixB.b
    }
}

let getPixelBilinear = function (image, x, y) {
    let x0 = Math.trunc(x);
    let y0 = Math.trunc(y);

    if (y0 != y) {
        let Y0 = getPixelBilinear(image, x, y0);
        let Y1 = getPixelBilinear(image, x, y0 + 1);
        return pixelAdd(pixelScale(Y0, y0 + 1 - y), pixelScale(Y1, y - y0));
    }

    if (x0 != x) {
        let X0 = getPixelBilinear(image, x0, y);
        let X1 = getPixelBilinear(image, x0 + 1, y);
        return pixelAdd(pixelScale(X0, x0 + 1 - x), pixelScale(X1, x - x0));
    }

    let rgb = {
        r: 0,
        g: 0,
        b: 0
    }

    if (x >= 0 && x <= image.bitmap.width - 1 && y >= 0 && y <= image.bitmap.height - 1) {
        let pix = Jimp.intToRGBA(image.getPixelColor(x, y));
        rgb.r = pix.r;
        rgb.g = pix.g;
        rgb.b = pix.b;
    }

    return rgb;
}

module.exports.calculateIntensity = function (imgModel, specWidth, callback) {
    let intensity = {
        nm: [],
        r: [],
        g: [],
        b: [],
        sum: []
    };

    Jimp.read(imgModel.get("path"), function (err, img) {
        let blueX = imgModel.get("blueX");
        let blueY = imgModel.get("blueY");
        let redX = imgModel.get("redX");
        let redY = imgModel.get("redY");
        let blueNM = imgModel.get("blueNM");
        let redNM = imgModel.get("redNM");

        if (blueNM > redNM) {
            callback(intensity);
            return;
        }

        let len = redNM - blueNM;
        let dx = (redX - blueX) / len;
        let dy = (redY - blueY) / len;

        let theta = Math.atan2(redY - blueY, redX - blueX) + (Math.PI / 2.0);
        let dxPix = Math.cos(theta);
        let dyPix = Math.sin(theta);

        blueX += ((-dxPix / 2) * (specWidth - 1));
        blueY += ((-dyPix / 2) * (specWidth - 1));
        redX += ((-dxPix / 2) * (specWidth - 1));
        redY += ((-dyPix / 2) * (specWidth - 1));

        for (let i = 0; i <= len; i++) {
            let nm = blueNM + i;
            intensity.nm[i] = nm;
            intensity.r[i] = 0.0;
            intensity.g[i] = 0.0;
            intensity.b[i] = 0.0;
            intensity.sum[i] = 0.0;

            for (let k = 0; k < specWidth; k++) {
                let x = blueX + (i * dx) + (k * dxPix);
                let y = blueY + (i * dy) + (k * dyPix);
                let px = getPixelBilinear(img, x, y);

                intensity.r[i] += px.r;
                intensity.g[i] += px.g;
                intensity.b[i] += px.b;
            }

            intensity.r[i] = Math.round(intensity.r[i]);
            intensity.g[i] = Math.round(intensity.g[i]);
            intensity.b[i] = Math.round(intensity.b[i]);
            intensity.sum[i] = intensity.r[i] + intensity.g[i] + intensity.b[i];
        }

        callback(intensity);
      
    });
}

module.exports.calculateTransmittance = function (smpIntModel, refIntModel) {
    let smpNM = smpIntModel.get("nm");
    let smpP = smpIntModel.get("sum");
    let refNM = refIntModel.get("nm");
    let refP = refIntModel.get("sum");

    let transmittance = {
        nm: [],
        t: [],
        a: []
    };
    if (smpNM.length == 0 || refNM.length == 0) {
        return transmittance;
    }

    let nmMin = Math.max(smpNM[0], refNM[0]);
    let nmMax = Math.min(smpNM[smpNM.length - 1], refNM[refNM.length - 1]);
    if (nmMin > nmMax) {
        return transmittance;
    }

    let smpOffset = nmMin - smpNM[0];
    let refOffset = nmMin - refNM[0];

    for (let nm = nmMin; nm <= nmMax; nm++) {
        let i = nm - nmMin;
        transmittance.nm[i] = nm;

        let t = smpP[i + smpOffset] / refP[i + refOffset];
        let a = -Math.log(t);
        transmittance.t[i] = Math.round(1000 * t) / 1000;
        transmittance.a[i] = Math.round(1000 * a) / 1000;
    }

    return transmittance;
};
